"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var types = require("@nativescript/core/utils/types");
var __1 = require("..");
var XMLHttpRequestResponseType;
(function (XMLHttpRequestResponseType) {
    XMLHttpRequestResponseType.empty = "";
    XMLHttpRequestResponseType.text = "text";
    XMLHttpRequestResponseType.json = "json";
    XMLHttpRequestResponseType.blob = "blob";
    XMLHttpRequestResponseType.arraybuffer = "arraybuffer";
})(XMLHttpRequestResponseType || (XMLHttpRequestResponseType = {}));
var NSHTTPXMLHttpRequest = (function () {
    function NSHTTPXMLHttpRequest() {
        this.UNSENT = 0;
        this.OPENED = 1;
        this.HEADERS_RECEIVED = 2;
        this.LOADING = 3;
        this.DONE = 4;
        this._responseType = "";
        this._listeners = new Map();
        this._readyState = this.UNSENT;
    }
    Object.defineProperty(NSHTTPXMLHttpRequest.prototype, "upload", {
        get: function () {
            return this;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NSHTTPXMLHttpRequest.prototype, "readyState", {
        get: function () {
            return this._readyState;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NSHTTPXMLHttpRequest.prototype, "responseType", {
        get: function () {
            return this._responseType;
        },
        set: function (value) {
            if (value === XMLHttpRequestResponseType.empty
                || value in XMLHttpRequestResponseType) {
                this._responseType = value;
            }
            else {
                throw new Error("Response type of '" + value + "' not supported.");
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NSHTTPXMLHttpRequest.prototype, "responseText", {
        get: function () {
            if (this._responseType !== XMLHttpRequestResponseType.empty
                && this._responseType !== XMLHttpRequestResponseType.text) {
                throw new Error("Failed to read the 'responseText' property from 'XMLHttpRequest': " +
                    "The value is only accessible if the object's 'responseType' is '' or 'text' " +
                    ("(was '" + this._responseType + "')."));
            }
            if (types.isFunction(this._responseTextReader)) {
                return this._responseTextReader();
            }
            return "";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NSHTTPXMLHttpRequest.prototype, "response", {
        get: function () {
            if (this._responseType === XMLHttpRequestResponseType.empty
                || this._responseType === XMLHttpRequestResponseType.text) {
                if (this._readyState !== this.LOADING && this._readyState !== this.DONE) {
                    return "";
                }
                else {
                    return this._response;
                }
            }
            else {
                if (this._readyState !== this.DONE) {
                    return null;
                }
                else {
                    return this._response;
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NSHTTPXMLHttpRequest.prototype, "status", {
        get: function () {
            return this._status;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NSHTTPXMLHttpRequest.prototype, "statusText", {
        get: function () {
            if (this._readyState === this.UNSENT
                || this._readyState === this.OPENED
                || this._errorFlag) {
                return "";
            }
            return statuses[this._status];
        },
        enumerable: true,
        configurable: true
    });
    NSHTTPXMLHttpRequest.prototype._loadResponse = function (r) {
        this._status = r.statusCode;
        this._headers = r.headers;
        this._setReadyState(this.HEADERS_RECEIVED);
        this._setReadyState(this.LOADING);
        this._responseTextReader = function () { return r.content.toString(); };
        var contentType = this.getResponseHeader("Content-Type");
        var mimeType = (contentType && contentType.toLowerCase()) || "text/xml";
        var finalMimeType = this._overrideMimeType || mimeType;
        if (this._responseType === XMLHttpRequestResponseType.json) {
            this._response = r.content.toJSON();
        }
        else if (this._responseType === XMLHttpRequestResponseType.text
            || this._responseType === XMLHttpRequestResponseType.empty) {
            this._response = this.responseText;
        }
        else if (this._responseType === XMLHttpRequestResponseType.arraybuffer) {
            this._response = r.content.toArrayBuffer();
        }
        else if (this._responseType === XMLHttpRequestResponseType.blob) {
            this._response = new Blob([r.content.toArrayBuffer()], { type: finalMimeType });
        }
        this.emitEvent("progress");
        this._sendFlag = false;
        this._setReadyState(this.DONE);
    };
    NSHTTPXMLHttpRequest.prototype.emitEvent = function (eventName) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (types.isFunction(this["on" + eventName])) {
            this["on" + eventName].apply(this, args);
        }
        var handlers = this._listeners.get(eventName) || [];
        handlers.forEach(function (handler) {
            handler.apply(void 0, args);
        });
    };
    NSHTTPXMLHttpRequest.prototype._setReadyState = function (value) {
        if (this._readyState !== value) {
            this._readyState = value;
            this.emitEvent("readystatechange");
        }
        if (this._readyState === this.DONE) {
            this.emitEvent("load");
            this.emitEvent("loadend");
        }
    };
    NSHTTPXMLHttpRequest.prototype._setRequestError = function (eventName, error) {
        this._readyState = this.DONE;
        this._response = error;
        this.emitEvent("readystatechange");
        this.emitEvent(eventName, error);
        this.emitEvent("loadend");
    };
    NSHTTPXMLHttpRequest.prototype.addEventListener = function (eventName, handler) {
        if (["abort", "error", "load", "loadend", "loadstart", "progress", "readystatechange"].indexOf(eventName) === -1) {
            throw new Error("Event not supported: " + eventName);
        }
        var handlers = this._listeners.get(eventName) || [];
        handlers.push(handler);
        this._listeners.set(eventName, handlers);
    };
    NSHTTPXMLHttpRequest.prototype.removeEventListener = function (eventName, toDetach) {
        var handlers = this._listeners.get(eventName) || [];
        handlers = handlers.filter(function (handler) { return handler !== toDetach; });
        this._listeners.set(eventName, handlers);
    };
    NSHTTPXMLHttpRequest.prototype.open = function (method, url, async, user, password) {
        if (types.isString(method) && types.isString(url)) {
            this._options = { url: url, method: method };
            this._options.headers = {};
            if (types.isString(user)) {
                this._options.headers["user"] = user;
            }
            if (types.isString(password)) {
                this._options.headers["password"] = password;
            }
            this._setReadyState(this.OPENED);
        }
    };
    NSHTTPXMLHttpRequest.prototype.abort = function () {
        this._response = null;
        this._responseTextReader = null;
        this._headers = null;
        this._status = null;
        if ((this._readyState === this.OPENED && this._sendFlag)
            || this._readyState === this.HEADERS_RECEIVED
            || this._readyState === this.LOADING) {
            this._errorFlag = true;
            this._sendFlag = false;
            this._setRequestError("abort");
        }
        if (this._readyState === this.DONE) {
            this._readyState = this.UNSENT;
        }
    };
    NSHTTPXMLHttpRequest.prototype.send = function (data) {
        var _this = this;
        this._errorFlag = false;
        this._response = null;
        this._responseTextReader = null;
        this._headers = null;
        this._status = null;
        if (this._readyState !== this.OPENED || this._sendFlag) {
            throw new Error("Failed to execute 'send' on 'XMLHttpRequest': " +
                "The object's state must be OPENED.");
        }
        if (types.isString(data) && this._options.method !== "GET") {
            this._options.content = data;
        }
        else if (data instanceof FormData) {
            this._options.content = data.toString();
        }
        else if (data instanceof Blob) {
            this.setRequestHeader("Content-Type", data.type);
            this._options.content = Blob.InternalAccessor.getBuffer(data);
        }
        else if (data instanceof ArrayBuffer) {
            this._options.content = data;
        }
        this._sendFlag = true;
        this.emitEvent("loadstart");
        __1.request(this._options).then(function (r) {
            if (!_this._errorFlag && _this._sendFlag) {
                _this._loadResponse(r);
            }
        }).catch(function (e) {
            _this._errorFlag = true;
            _this._sendFlag = false;
            _this._setRequestError("error", e);
        });
    };
    NSHTTPXMLHttpRequest.prototype.setRequestHeader = function (header, value) {
        if (this._readyState !== this.OPENED || this._sendFlag) {
            throw new Error("Failed to execute 'setRequestHeader' on 'XMLHttpRequest': " +
                "The object's state must be OPENED.");
        }
        if (types.isString(header) && types.isString(value)) {
            this._options.headers[header] = value;
        }
    };
    NSHTTPXMLHttpRequest.prototype.getAllResponseHeaders = function () {
        if (this._readyState < 2 || this._errorFlag) {
            return "";
        }
        var result = "";
        for (var i in this._headers) {
            result += i + ": " + this._headers[i] + "\r\n";
        }
        return result.substr(0, result.length - 2);
    };
    NSHTTPXMLHttpRequest.prototype.getResponseHeader = function (header) {
        if (types.isString(header) && this._readyState > 1
            && this._headers
            && !this._errorFlag) {
            header = header.toLowerCase();
            for (var i in this._headers) {
                if (i.toLowerCase() === header) {
                    return this._headers[i];
                }
            }
        }
        return null;
    };
    NSHTTPXMLHttpRequest.prototype.overrideMimeType = function (mime) {
        if (this._readyState === this.LOADING || this._readyState === this.DONE) {
            throw new Error("Failed to execute 'overrideMimeType' on 'XMLHttpRequest': " +
                "MimeType cannot be overridden when the state is LOADING or DONE.");
        }
        this._overrideMimeType = mime;
    };
    return NSHTTPXMLHttpRequest;
}());
exports.NSHTTPXMLHttpRequest = NSHTTPXMLHttpRequest;
var statuses = {
    100: "Continue",
    101: "Switching Protocols",
    200: "OK",
    201: "Created",
    202: "Accepted",
    203: "Non - Authoritative Information",
    204: "No Content",
    205: "Reset Content",
    206: "Partial Content",
    300: "Multiple Choices",
    301: "Moved Permanently",
    302: "Found",
    303: "See Other",
    304: "Not Modified",
    305: "Use Proxy",
    307: "Temporary Redirect",
    400: "Bad Request",
    401: "Unauthorized",
    402: "Payment Required",
    403: "Forbidden",
    404: "Not Found",
    405: "Method Not Allowed",
    406: "Not Acceptable",
    407: "Proxy Authentication Required",
    408: "Request Timeout",
    409: "Conflict",
    410: "Gone",
    411: "Length Required",
    412: "Precondition Failed",
    413: "Request Entity Too Large",
    414: "Request - URI Too Long",
    415: "Unsupported Media Type",
    416: "Requested Range Not Satisfiable",
    417: "Expectation Failed",
    500: "Internal Server Error",
    501: "Not Implemented",
    502: "Bad Gateway",
    503: "Service Unavailable",
    504: "Gateway Timeout",
    505: "HTTP Version Not Supported"
};
var NSHTTPXhr = (function () {
    function NSHTTPXhr() {
    }
    NSHTTPXhr.prototype.build = function () {
        return (new NSHTTPXMLHttpRequest());
    };
    NSHTTPXhr = __decorate([
        core_1.Injectable(),
        __metadata("design:paramtypes", [])
    ], NSHTTPXhr);
    return NSHTTPXhr;
}());
exports.NSHTTPXhr = NSHTTPXhr;
//# sourceMappingURL=xhr.js.map