"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var http_android_1 = require("../http.android");
var WebsocketConnection = (function () {
    function WebsocketConnection(nativeConnection) {
        this.nativeConnection = nativeConnection;
    }
    WebsocketConnection.prototype.queueSize = function () {
        return this.nativeConnection.queueSize();
    };
    WebsocketConnection.prototype.send = function (text) {
        this.nativeConnection.send(text);
    };
    WebsocketConnection.prototype.sendBinary = function (bytes) {
        var typedArray = new Uint8Array(bytes);
        var nativeBuffer = java.nio.ByteBuffer.wrap(Array.from(typedArray));
        var nativeByteString = okio.ByteString.of(nativeBuffer);
        this.nativeConnection.send(nativeByteString);
    };
    WebsocketConnection.prototype.close = function (code, reason) {
        return this.nativeConnection.close(code, reason);
    };
    WebsocketConnection.prototype.cancel = function () {
        this.nativeConnection.cancel();
    };
    return WebsocketConnection;
}());
exports.WebsocketConnection = WebsocketConnection;
function newWebsocketConnection(options, callbacks) {
    return new Promise(function (resolve, reject) {
        try {
            var javaOptions = http_android_1.buildJavaOptions(options);
            var OurListener = (function (_super) {
                __extends(OurListener, _super);
                function OurListener() {
                    return _super !== null && _super.apply(this, arguments) || this;
                }
                OurListener.prototype.onClosed = function (ws, code, reason) {
                    callbacks.onClosed(code, reason);
                };
                OurListener.prototype.onMessage = function (ws, data) {
                    if (typeof data === "string") {
                        callbacks.onMessage(data);
                    }
                    else {
                        var arrayBuffer = new Uint8Array(data.toByteArray()).buffer;
                        callbacks.onBinaryMessage(arrayBuffer);
                    }
                };
                OurListener.prototype.onFailure = function (ws, t, response) {
                    callbacks.onFailure(t.getMessage());
                };
                OurListener.prototype.onOpen = function (ws, response) {
                    callbacks.onOpen();
                };
                OurListener.prototype.onClosing = function (ws, code, reason) {
                    callbacks.onClosing(code, reason);
                };
                return OurListener;
            }(okhttp3.WebSocketListener));
            var listener = new OurListener();
            var websocket = com.klippa.NativeScriptHTTP.Async.Http.GetWebSocketConnection(javaOptions, listener);
            var websocketConnection = new WebsocketConnection(websocket);
            resolve(websocketConnection);
        }
        catch (e) {
            reject(e);
        }
    });
}
exports.newWebsocketConnection = newWebsocketConnection;
//# sourceMappingURL=index.android.js.map